//============================================================================
// Name        : upower22_api.h
// Author      : JerryLiao
// Version     : 0.1
// Date		   : 2020/10/13
// Copyright   : Copyright © 2020 BY Onyx Healthcare Inc.
// Description : UPower pro 22 library header file
//============================================================================

#define OnyxApiStatus int

/**
 *
 *  Onyx SDK status code
 *
 */
#define STATUS_SUCCESS				0
#define STATUS_FAILED				1
#define STATUS_INVALID_PARAMETER	2
#define STATUS_NOT_SUPPORT			3

struct BatteryData
{
    unsigned char ReportID;
    unsigned short BatteryMode;
    unsigned short Temperature;
    unsigned short Voltage;
    short Current;
    short AverageCurrent;
    unsigned short RemainPercentage;
    unsigned short RemainingCapacity;
    unsigned short FullChargeCapacity;
    unsigned short AverageTimeToEmpty;
    unsigned short AverageTimeToFull;
    short ChargingCurrent;
    unsigned short ChargingVoltage;
    unsigned short GaugeStatus;
    unsigned short CycleCount;
    unsigned short DesignCapacity;
    unsigned short DesignVoltage;
    unsigned short ManufactureDate;
    char SerialNumber[22];
    unsigned char Status;

    // XXL Battery Extend Data
    unsigned short LogIndex;
    char FullyChargeDate[8];
    unsigned short FccThreshold;
    unsigned short FccBackup;
    unsigned short CycleCountThreshold;
    unsigned short LifetimeMaxPackVoltage;
    unsigned short LifetimeMinPackVoltage;
    short LifetimeMaxCurrent;
    short LifetimeMinCurrent;
    unsigned short SmbLedBlink;
    unsigned short FullChargeVoltage;
    unsigned short DateRecord;
    unsigned short TimerRecord;
    unsigned short BatteryFirstUseTime;
    unsigned short PackMode;
};

extern "C" 
{
/**
 * upowerApiInitialize
 *
 * @brief	Initialize the API
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerApiInitialize();

/**
 * upowerApiUninitialize
 *
 * @brief	Please use this API before close the process
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerApiUninitialize();

/**
 * upowerLogControl
 *
 * @brief	Log function initialization
 *          It will generate the log file under /tmp/upower_sdk.log
 *
 * @param	bEnable: Enable the setting
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerLogControl(bool bEnable);

/**
 * upowerGetProductName
 *
 * @brief	Get product name
 *
 * @retval	strProduct: String for product name
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetProductName(char *strProduct);

/**
 * upowerGetFirmwareVersion
 *
 * @brief	Get firmware version
 *
 * @retval	strVersion: String for firmware version
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetFirmwareVersion(char *strVersion);

/**
 * upowerGetSerialNumber
 *
 * @brief	Get UPower serial number
 *
 * @retval	strSerialNumber: String for serial number
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetSerialNumber(char *strSerialNumber);

/**
 * upowerSetSerialNumber
 *
 * @brief	Set UPower serial number.
 *			It takes effect after device reboot
 *
 * @param	strSerialNumber: String for serial number
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerSetSerialNumber(char *strSerialNumber);

/**
 * upowerRebootDevice
 *
 * @brief	Reboot the device
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerRebootDevice();

/**
 * upowerGetACStatus
 *
 * @brief	Check the AC power is plug in or not
 *
 * @retval	bIsPlugIn: true: plug in, false: plug out
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetACStatus(bool* bIsPlugIn);

/**
 * upowerGetDCOutVoltage
 *
 * @brief	Get DC output voltage of UPower
 *
 * @retval	nVoltage: Output voltage
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetDCOutVoltage(int* nVoltage);

/**
 * upowerGetDCOutStatus
 *
 * @brief	Get DC output status of UPower
 *
 * @retval	anStatus[0]: Output voltage select.
 * @retval	anStatus[1]: DC output cable connection. 1: Connect, 0, Disconnect
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetDCOutStatus(int* anStatus);

/**
 * upowerGetBoardId
 *
 * @brief	Get Board Id of UPower
 *
 * @retval	nBoardId: Board Id
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetBoardId(int* nBoardId);

/**
 * upowerGetTotalPercentage
 *
 * @brief	Get total percentage of batteries
 *
 * @retval	nPercentage: Remain total percentage of batteries
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetTotalPercentage(int* nPercentage);

/**
 * upowerSetLedColor
 *
 * @brief	Set the LED color on UPower
 *
 * @retval	nIndex: Index of LED
 * @retval	Red: 0~255
 * @retval	Green: 0~255
 * @retval	Blue: 0~255
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerSetLedColor(int nIndex, int Red, int Green, int Blue);

/**
 * upowerResetLedColor
 *
 * @brief	Reset the LED to previous status
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerResetLedColor();

/**
 * upowerGetBatteryInfo
 *
 * @brief	Get batteries information
 *
 * @param	nBatteryNumber: Index of battery
 *
 * @retval	BatteryData: Structure of battery information
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerGetBatteryInfo(int nBatteryNumber, BatteryData* rawData);


/**
 * upowerUpdateFirmware
 *
 * @brief	Update UPower firmware version
 *
 * @param	filename: Firmware file path
 * @param	bForceUpdate: true: Ignore firmware version confirmation
 *
 * @return	Please refer to Onyx SDK status code
 *
 */
OnyxApiStatus upowerUpdateFirmware(char* filename, bool bForceUpdate);
}